@@ -12,6 +12,9 @@ use once_cell::sync::Lazy;
 use bytes::Buf;
 use bytes::BufMut;
 
+use crate::batch::BatchRecords;
+use crate::batch::MemoryRecords;
+use crate::batch::RawRecords;
 use crate::core::{Encoder, Decoder};
 use crate::core::DecoderVarInt;
 use crate::core::EncoderVarInt;
@@ -19,6 +22,7 @@ use crate::core::Version;
 
 use crate::batch::Batch;
 use crate::Offset;
+use fluvio_compression::CompressionError;
 
 /// maximum text to display
 static MAX_STRING_DISPLAY: Lazy<usize> = Lazy::new(|| {
@@ -203,8 +207,20 @@ impl Decoder for RecordData {
 /// Represents sets of batches in storage
 //  It is written consequently with len as prefix
 #[derive(Default, Debug)]
-pub struct RecordSet {
-    pub batches: Vec<Batch>,
+pub struct RecordSet<R = MemoryRecords> {
+    pub batches: Vec<Batch<R>>,
+}
+
+impl TryFrom<RecordSet> for RecordSet<RawRecords> {
+    type Error = CompressionError;
+    fn try_from(set: RecordSet) -> Result<Self, Self::Error> {
+        let batches: Result<Vec<_>, _> = set
+            .batches
+            .into_iter()
+            .map(|batch| batch.try_into())
+            .collect();
+        Ok(Self { batches: batches? })
+    }
 }
 
 impl fmt::Display for RecordSet {
@@ -213,8 +229,8 @@ impl fmt::Display for RecordSet {
     }
 }
 
-impl RecordSet {
-    pub fn add(mut self, batch: Batch) -> Self {
+impl<R: BatchRecords> RecordSet<R> {
+    pub fn add(mut self, batch: Batch<R>) -> Self {
         self.batches.push(batch);
         self
     }
@@ -230,7 +246,7 @@ impl RecordSet {
     pub fn total_records(&self) -> usize {
         self.batches
             .iter()
-            .map(|batches| batches.records().len())
+            .map(|batches| batches.records_len())
             .sum()
     }
 
@@ -243,7 +259,7 @@ impl RecordSet {
     }
 }
 
-impl Decoder for RecordSet {
+impl<R: BatchRecords> Decoder for RecordSet<R> {
     fn decode<T>(&mut self, src: &mut T, version: Version) -> Result<(), Error>
     where
         T: Buf,
@@ -296,7 +312,7 @@ impl Decoder for RecordSet {
     }
 }
 
-impl Encoder for RecordSet {
+impl<R: BatchRecords> Encoder for RecordSet<R> {
     fn write_size(&self, version: Version) -> usize {
         self.batches
             .iter()
@@ -613,7 +629,8 @@ mod test {
         assert_eq!(out.len(), original_len - TRUNCATED);
 
         println!("decoding...");
-        let decoded_batches = RecordSet::decode_from(&mut Cursor::new(out), 0).expect("decoding");
+        let decoded_batches =
+            RecordSet::<MemoryRecords>::decode_from(&mut Cursor::new(out), 0).expect("decoding");
         assert_eq!(decoded_batches.batches.len(), 2);
     }
 
